@@ -57,14 +57,19 @@ eem_parafac <- function(eem_list, comps, maxit = 2500, normalise = TRUE, const =
     eem_array <- eem_array %>% norm_array()
   }
   if(verbose) cat(paste0(cores," core",ifelse(cores > 1,"s are", " is" )," used for the calculation."),fill=TRUE)
+
+  cl <- NULL
+  if(cores > 1){
+    cl <- makeCluster(min(cores,nstart), type="PSOCK")
+    clusterExport(cl, c("eem_array","maxit","const","ctol","cores"), envir=environment())
+    clusterEvalQ(cl, library(multiway))
+  }
+
   res <- lapply(comps,function(comp){
     #comp <- 6
     if(verbose) cat(paste0("calculating ",comp," components model..."),fill=TRUE)
-    cl <- NULL
     if(cores > 1){
-      cl <- makeCluster(min(cores,nstart), type="PSOCK")
-      clusterExport(cl, c("eem_array","comp","maxit","const","ctol","cores"), envir=environment())
-      clusterEvalQ(cl, library(multiway))
+      clusterExport(cl, c("comp"), envir=environment())
     }
     if(strictly_converging){
       cpresult <- parafac_conv(eem_array, nfac = comp, const = const, maxit = maxit, parallel = (cores > 1), cl = cl, ctol = ctol, nstart = nstart, output = "all", verbose = verbose, ...)
@@ -73,9 +78,6 @@ eem_parafac <- function(eem_list, comps, maxit = 2500, normalise = TRUE, const =
     }
     Rsqs <- lapply(cpresult,`[[`,"Rsq") %>% unlist()
     cpresult1 <- cpresult[[which.max(Rsqs)]]
-    if(cores > 1){
-      stopCluster(cl)
-    }
     cflags <- lapply(cpresult,`[[`,"cflag") %>% unlist()
     converged <- sum(cflags == 0)/nstart
     if(converged <= 0.5 & sum(cflags == 0) < 20){
@@ -91,6 +93,9 @@ eem_parafac <- function(eem_list, comps, maxit = 2500, normalise = TRUE, const =
     cpresult1$converged <- converged
     cpresult1
   })
+  if(cores > 1){
+    stopCluster(cl)
+  }
   mostattributes(res) <- attributes(eem_array)
   return(res)
 }
@@ -415,7 +420,7 @@ eempf_leverage <- function(pfmodel){
 
 #' Calculate the leverage of each emission and excitation wavelength and each sample from a list of PARAFAC models
 #'
-#' @param pfres_comps object of class parafac
+#' @param pfres_comps list of one or more parafac models
 #' @param ecdf logical, transforme leverages to according empirical quantiles (\code{\link[stats]{ecdf}})
 #' @param stats logical, whether means and standard deviations are calculated from leverages
 #'
@@ -430,7 +435,7 @@ eempf_leverage <- function(pfmodel){
 #' data(pf_models)
 #'
 #' eempf_mleverage(pf3)
-eempf_mleverage <- function(pfres_comps,ecdf = FALSE, stats = FALSE){
+eempf_mleverage <- function(pfres_comps, ecdf = FALSE, stats = FALSE){
   cpls <- lapply(pfres_comps,eempf_leverage) %>%
     lapply(unlist) %>%
     lapply(function(ll) data.frame(parameter=names(ll),value=ll)) %>%
@@ -545,9 +550,9 @@ eempf_cortable <- function(pfmodel,normalisation = FALSE, method="pearson",...){
 #' ml <- maxlines(pf4[[1]])
 maxlines <- function(pfmodel){
   maxl <- lapply(colnames(pfmodel$C),function(comp){
-    em = (pfmodel$B[,comp]*max(pfmodel$C[,comp])) %>%
+    em <- (pfmodel$B[,comp]*max(pfmodel$C[,comp])) %>%
       data.frame(e="em", wavelength = as.numeric(names(.)),value=.)
-    ex = (pfmodel$C[,comp]*max(pfmodel$B[,comp])) %>%
+    ex <- (pfmodel$C[,comp]*max(pfmodel$B[,comp])) %>%
       data.frame(e="ex", wavelength = as.numeric(names(.)),value=.)
     res <- bind_rows(em,ex) %>%
       setNames(c("e","wavelength",comp))
@@ -700,12 +705,13 @@ eempf_residuals_metrics <- function(residuals, leverage){
 #' @param eem_list object of class eemlist with sample data
 #' @param pfmodel object of class parafac
 #' @param cores number of cores to use for parallel processing
+#' @param adj_samples logical, whether wavelengths in the samples are cut automatically to fit the present data in the model
 #' @param components optionally supply components to use manually, either as a variable of class parafac_components or as a list of variables of class parafac_components, if you do so,
 #' @param const optional constraints for model, just used, when components are supplied
 #' @param control optional constraint control parameters for model, just used, when components are supplied
 #' @param ... additional arguments passed to eem_parafac
 #'
-#' @details This function can be used to calculate A modes (sample loadings) for samples that were previously excluded from the modelling process (e.g. outliers). Another way to use it would be a recombination of components from different models and calculating the according sample loadings. Expecially the later application is experimental and results have to be seen critically! Nevertheless, I decided to supply this function to stimulate some experiments on that and would be interested in your findings and feedback.
+#' @details This function can be used to calculate A modes (sample loadings) for samples that were previously excluded from the modelling process (e.g. outliers). The wavelengths in the samples and in the PARAFAC model must be identically. Surplus wavelengths in the samples can be adjusted automatically with the argument adj_samples = TRUE, surplus wavelengths in the PARAFAC model have to be mitigated by interpolating the missing wavelengths in the samples manually. If there is a mismatch in wavelengths, the function provides you with the necessary information. Another way to use it would be a recombination of components from different models and calculating the according sample loadings. Expecially the later application is experimental and results have to be seen critically! Nevertheless, I decided to supply this function to stimulate some experiments on that and would be interested in your findings and feedback.
 #'
 #' @return object of class parafac
 #' @export
@@ -721,17 +727,58 @@ eempf_residuals_metrics <- function(residuals, leverage){
 #' data(pf_models)
 #'
 #' A_missing(eem_list, pf4[[1]], cores = 2)
+#'
+#'
 #' }
-A_missing <- function(eem_list,pfmodel = NULL,cores = parallel::detectCores(logical = FALSE),components = NULL, const = NULL, control = NULL, ...){
-  eem_list <- eem_red2smallest(eem_list)
+A_missing <- function(eem_list, pfmodel = NULL, cores = parallel::detectCores(logical = FALSE), adj_samples = TRUE, components = NULL, const = NULL, control = NULL, ...){
   if(is.null(pfmodel) & is.null(components)) stop("You must either specify a model or components as a base for the newly generated model!")
 
   exclude <- list("ex" = eem_list[[1]]$ex[!(eem_list[[1]]$ex %in% rownames(pfmodel$C))],
                   "em" = eem_list[[1]]$em[!(eem_list[[1]]$em %in% rownames(pfmodel$B))],
                   "sample" = c()
   )
-  x <- eem_list %>%
-    eem_exclude(exclude)
+
+  red_list <- eem_red2smallest(eem_list, return = "list")
+
+  if(adj_samples) {
+    if(length(red_list$ex)>0 | length(red_list$em)>0){
+      eem_list <- eem_red2smallest(eem_list)
+
+      warning("The wavelengths in the provided samples are not equal over the whole sample set and were automatically removed.")
+    }
+    if(!is.null(exclude$ex) | !is.null(exclude$em)){
+      x <- eem_list %>%
+        eem_exclude(exclude)
+      warning(paste0("The provided samples contain the following wavelengths that
+            are not present in the PARAFAC model: Emission ",
+                     paste0(exclude$em, collapse = ", "),
+                     "; Excitation ",
+                     paste0(exclude$ex, collapse = ", "), ". These wavelengths were automatically removed."))
+    }
+  } else {
+    if(length(red_list$ex)>0 | length(red_list$em)>0){
+      stop("The wavelengths in the provided samples are not equal over the whole sample set. Please run A_missing with adj_samples = TRUE to remove the surplus wavelengths automatically or adjust manually.")
+    }
+    if(!is.null(exclude$ex) | !is.null(exclude$em)){
+      stop(paste0("The provided samples contain the following wavelengths that
+            are not present in the PARAFAC model: Emission ",
+                  paste0(exclude$em, collapse = ", "),
+                  "; Excitation ",
+                  paste0(exclude$ex, collapse = ", "), ". These wavelengths can be excluded automatically, if A_missing is run with adj_samples = TRUE."))
+    }
+  }
+
+  missing <- list("ex" = rownames(pfmodel$C)[(!rownames(pfmodel$C) %in% eem_list[[1]]$ex)],
+                  "em" = rownames(pfmodel$B)[(!rownames(pfmodel$B) %in% eem_list[[1]]$em)])
+
+  if(length(missing$ex) > 0 | length(missing$em) > 0){
+    stop(paste0("The provided samples are missing the following wavelengths that
+            are present in the PARAFAC model: Emission ",
+                paste0(missing$em, collapse = ", "),
+                "; Excitation ",
+                paste0(missing$ex, collapse = ", "), "."))
+  }
+
   if(!is.null(components)){
     if(!is.null(pfmodel)) warning("The base model is ignored since you provided components manually!")
     if(inherits(components[[1]], "parafac_components")) components <- eempf_bindxc(components)
@@ -747,9 +794,9 @@ A_missing <- function(eem_list,pfmodel = NULL,cores = parallel::detectCores(logi
     Bfixed <- pfmodel$B
     Cfixed <- pfmodel$C
     comps <- pfmodel$A %>% ncol()
-    normalise = (!is.null(attr(pfmodel,"norm_factors")))
-    control = pfmodel$control
-    const = pfmodel$const
+    normalise <- (!is.null(attr(pfmodel,"norm_factors")))
+    control <- pfmodel$control
+    const <- pfmodel$const
   }
   missingAs <- eem_parafac(x,comps = comps,normalise = (!is.null(attr(pfmodel,"norm_factors"))),Bfixed = Bfixed, Cfixed = Cfixed,cores = cores,const = const, control = control, ...)
   missingAs[[1]]
@@ -828,7 +875,7 @@ splithalf <- function(eem_list, comps, splits = NA, rand = FALSE, normalise = TR
 
   if(verbose) close(pb)
 
-#  if(verbose) cat("The returned fits variable is of class", class(fits), ", subclasses are [", paste(lapply(fits, lapply, class) %>% unlist(), collapse = ","), "] and has", length(fits), "elements that are named [", paste(names(fits), collapse = ","), "].", fill = TRUE)
+  #  if(verbose) cat("The returned fits variable is of class", class(fits), ", subclasses are [", paste(lapply(fits, lapply, class) %>% unlist(), collapse = ","), "] and has", length(fits), "elements that are named [", paste(names(fits), collapse = ","), "].", fill = TRUE)
 
   sscs <- eempf_ssc(fits, tcc = TRUE, cores = cores)
 
@@ -1097,9 +1144,9 @@ eempf_openfluor <- function(pfmodel, file, Fmax = TRUE, upload = FALSE, email =
   }
   if(!is.null(email)){
     if(!grepl("\\<[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,}\\>",
-           as.character(email), ignore.case = TRUE)){
-             warning("The email address does not correspond to the expected pattern!")
-  }
+              as.character(email), ignore.case = TRUE)){
+      warning("The email address does not correspond to the expected pattern!")
+    }
   }
   if(Fmax){
     pfmodel <- eempf_rescaleBC(pfmodel)
@@ -1539,13 +1586,13 @@ eempf_ssc <- function(pfmodels, tcc = FALSE, m = FALSE, cores = parallel::detect
   } else if(all(classes == "matrix")){ ## matrices
     cl <- makePSOCKcluster(min(cores,length(pfmodels)))
     clusterExport(cl, c("pfmodels","tcc"), envir = environment())
-    clusterEvalQ(cl,require(staRdom))
+    clusterEvalQ(cl, require(staRdom))
 
     SSCs <- parLapply(cl,1:length(pfmodels),function(k){
       lapply(k:length(pfmodels), function(l){
-        B = ssc(pfmodels[[k]][[1]], pfmodels[[l]][[1]], tcc = tcc)
-        C = ssc(pfmodels[[k]][[2]], pfmodels[[l]][[2]], tcc = tcc)
-        list(B=B, C=C)
+        B <- ssc(pfmodels[[k]][[1]], pfmodels[[l]][[1]], tcc = tcc)
+        C <- ssc(pfmodels[[k]][[2]], pfmodels[[l]][[2]], tcc = tcc)
+        list(B = B, C = C)
       }) %>%
         setNames(paste0(k,"vs",k:length(pfmodels)))
     }) %>% unlist(recursive = FALSE)
@@ -1654,7 +1701,7 @@ eempf_ssccheck <- function(pfmodels, best = length(pfmodels), tcc = FALSE, cores
 
   cl <- makePSOCKcluster(min(cores, length(sscs)))
   clusterExport(cl, c("sscs"), envir=environment())
-  clusterEvalQ(cl,require(staRdom))
+  clusterEvalQ(cl, require(staRdom))
 
   maxs <- parLapply(cl, sscs, lapply, ssc_max)
 
@@ -1696,27 +1743,34 @@ eempf_ssccheck <- function(pfmodels, best = length(pfmodels), tcc = FALSE, cores
 #' attr(sscs,"order")
 ssc_max <- function(mat){
   n <- min(dim(mat))
-  p <- permutations(n , n)
-  combinations <- lapply(1:nrow(p),function(row){
-    per <- p[row,]
-    matrix(c(1:n,per), ncol = n,byrow = TRUE)
-  })
+  if(n > 1){
+    p <- permutations(n , n)
+    combinations <- lapply(1:nrow(p),function(row){
+      per <- p[row,]
+      matrix(c(1:n,per), ncol = n,byrow = TRUE)
+    })
 
-  best_comb <- lapply(combinations, function(c){
-    pair <- c[,2] %>% unlist()
-    apply(c,2,function(pair){
-      mat[pair[1],pair[2]] %>% `^`(2)
+    best_comb <- lapply(combinations, function(c){
+      pair <- c[,2] %>% unlist()
+      apply(c,2,function(pair){
+        mat[pair[1],pair[2]] %>% `^`(2)
+      }) %>%
+        sum() %>%
+        sqrt()
     }) %>%
-      sum() %>%
-      sqrt()
-  }) %>%
-    which.max() %>%
-    combinations[[.]]
+      which.max() %>%
+      combinations[[.]]
 
-  res <- apply(best_comb,2,function(pair){
-    mat[pair[1],pair[2]]
-  })
-  attr(res,"order") <- best_comb[2,]
+    res <- apply(best_comb,2,function(pair){
+      mat[pair[1],pair[2]]
+    })
+
+    attr(res,"order") <- best_comb[2,]
+
+  } else {
+    res <- c(1)
+    attr(res,"order") <- c(1)
+  }
   res
 }
 
@@ -1744,14 +1798,14 @@ eempf_convergence <- function(pfmodel, print = TRUE){
     conv <- lapply(pfmodel$models,`[[`,"cflag") %>% unlist()
     res <- list(models = n, converging = sum(conv == 0), nc_itlim = sum(conv == 1), nc_other = sum(conv == 2), conv = conv, sses = sses)
     if(print){
-    cat("Calculated models: ", n, fill = TRUE)
-    cat("Converging models: ", sum(conv == 0), fill = TRUE)
-    cat("Not converging Models, iteration limit reached: ", sum(conv == 1), fill = TRUE)
-    cat("Not converging models, other reasons: ", sum(conv == 2), fill = TRUE)
-    cat("Best SSE: ", min(sses), fill = TRUE)
-    cat("Summary of SSEs of converging models:",fill = TRUE)
-    summary(sses[conv == 0]) %>%
-      print()
+      cat("Calculated models: ", n, fill = TRUE)
+      cat("Converging models: ", sum(conv == 0), fill = TRUE)
+      cat("Not converging Models, iteration limit reached: ", sum(conv == 1), fill = TRUE)
+      cat("Not converging models, other reasons: ", sum(conv == 2), fill = TRUE)
+      cat("Best SSE: ", min(sses), fill = TRUE)
+      cat("Summary of SSEs of converging models:",fill = TRUE)
+      summary(sses[conv == 0]) %>%
+        print()
     }
     invisible(res)
   }
